/*
    DoubleSimple: DoubleError.m
    Copyright (C) 2003  Macaulay Institute

    This file is part of DoubleSimple, a series of classes for implementing
    various remedies to floating point arithmetic errors.

    DoubleSimple is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    DoubleSimple is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details. (LICENCE file in
    this directory.)

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Contact information:
      Gary Polhill,
      Macaulay Institute, Craigiebuckler, Aberdeen, AB15 8QH. United Kingdom
      g.polhill@macaulay.ac.uk
*/
/*

Implementation of DoubleError class

*/

#import "DoubleError.h"
#import <stdlib.h>

#define ERROR(type, method) \
  type retval = [super method: value]; \
  if(last_op_err) abort(); \
  return retval

@implementation DoubleError

-add: (DoubleSimple *)value {
  ERROR(id, add);
}

-sub: (DoubleSimple *)value {
  ERROR(id, sub);
}

-mul: (DoubleSimple *)value {
  ERROR(id, mul);
}

-div: (DoubleSimple *)value {
  ERROR(id, div);
}

-iadd: (int)value {
  ERROR(id, iadd);
}

-isub: (int)value {
  ERROR(id, isub);
}

-imul: (int)value {
  ERROR(id, imul);
}

-idiv: (int)value {
  ERROR(id, idiv);
}

-cadd: (double)value {
  ERROR(id, cadd);
}

-csub: (double)value {
  ERROR(id, csub);
}

-cmul: (double)value {
  ERROR(id, cmul);
}

-cdiv: (double)value {
  ERROR(id, cdiv);
}

-(BOOL)ieq: (int)value {
  ERROR(BOOL, ieq);
}

-(BOOL)ine: (int)value {
  ERROR(BOOL, ine);
}

-(BOOL)igt: (int)value {
  ERROR(BOOL, igt);
}

-(BOOL)ile: (int)value {
  ERROR(BOOL, ile);
}

-(BOOL)ige: (int)value {
  ERROR(BOOL, ige);
}

-(BOOL)ilt: (int)value {
  ERROR(BOOL, ilt);
}

-(double)checkConst: (const char *)value {
  ERROR(double, checkConst);
}

@end
